@@ -21,19 +21,17 @@
 #include <string.h>
 #include <errno.h>
 
+#include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
 #ifdef HAVE_GNOME
 #include <gconf/gconf-client.h>
-#include <libgnomeui/gnome-app-helper.h>
 #else
-#include <glib/gkeyfile.h>
-#include <gtk/gtkaccelmap.h>
-
 #define ACCELMAP_EXT "accels"
-
 #endif
 
+#include "games-debug.h"
+#include "games-gtk-compat.h"
 #include "games-marshal.h"
 
 #include "games-conf.h"
@@ -74,7 +72,7 @@ G_DEFINE_TYPE (GamesConf, games_conf, G_TYPE_OBJECT);
 
 /* helper functions */
 
-#define WINDOW_STATE_TIMEOUT 1000 /* ms */
+#define WINDOW_STATE_TIMEOUT 1 /* s */
 
 enum {
   STATE_KEY_MAXIMISED,
@@ -99,6 +97,7 @@ static const char window_state_key_name[][12] = {
 };
 
 typedef struct {
+  GtkWindow *window;
   char *group;
   guint timeout_id;
   int width;
@@ -113,6 +112,11 @@ window_state_timeout_cb (WindowState *state)
   games_conf_set_integer (state->group, window_state_key_name[STATE_KEY_WIDTH], state->width);
   games_conf_set_integer (state->group, window_state_key_name[STATE_KEY_HEIGHT], state->height);
 
+  _games_debug_print (GAMES_DEBUG_WINDOW_STATE,
+                      "[window %p] timeout: persisting width:%d height:%d\n",
+                      state->window,
+                      state->width, state->height);
+
   state->timeout_id = 0;
   return FALSE;
 }
@@ -129,7 +133,11 @@ free_window_state (WindowState *state)
 
   g_free (state->group);
 
+#if GLIB_CHECK_VERSION (2, 10, 0)
   g_slice_free (WindowState, state);
+#else
+  g_free (state);
+#endif
 }
 
 static gboolean
@@ -137,15 +145,33 @@ window_configure_event_cb (GtkWidget *widget,
                            GdkEventConfigure *event,
                            WindowState *state)
 {
+  _games_debug_print (GAMES_DEBUG_WINDOW_STATE,
+                      "[window %p] configure event current %dx%d new %dx%d [state: is-maximised:%s is-fullscreen:%s]\n",
+                      state->window,
+                      state->width, state->height,
+                      event->width, event->height,
+                      state->is_maximised ? "t" : "f",
+                      state->is_fullscreen ? "t" : "f");
+
   if (!state->is_maximised && !state->is_fullscreen &&
       (state->width != event->width || state->height != event->height)) {
     state->width = event->width;
     state->height = event->height;
 
+  _games_debug_print (GAMES_DEBUG_WINDOW_STATE,
+                      "[window %p] scheduling save of new window size\n",
+                      state->window);
+
     if (state->timeout_id == 0) {
-      state->timeout_id = g_timeout_add (WINDOW_STATE_TIMEOUT,
+#if GLIB_CHECK_VERSION (2, 14, 0)
+      state->timeout_id = g_timeout_add_seconds (WINDOW_STATE_TIMEOUT,
+                                                 (GSourceFunc) window_state_timeout_cb,
+                                                 state);
+#else
+      state->timeout_id = g_timeout_add (WINDOW_STATE_TIMEOUT * 1000,
                                          (GSourceFunc) window_state_timeout_cb,
                                          state);
+#endif
     }
   }
 
@@ -157,6 +183,13 @@ window_state_event_cb (GtkWidget *widget,
                        GdkEventWindowState *event,
                        WindowState *state)
 {
+  _games_debug_print (GAMES_DEBUG_WINDOW_STATE,
+                      "[window %p] state event, mask:%x new-state:%x current state: is-maximised:%s is-fullscreen:%s\n",
+                      state->window,
+                      event->changed_mask, event->new_window_state,
+                      state->is_maximised ? "t" : "f",
+                      state->is_fullscreen ? "t" : "f");
+
   if (event->changed_mask & GDK_WINDOW_STATE_MAXIMIZED) {
     state->is_maximised = (event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED) != 0;
     games_conf_set_boolean (state->group, window_state_key_name[STATE_KEY_MAXIMISED], state->is_maximised);
@@ -166,9 +199,93 @@ window_state_event_cb (GtkWidget *widget,
     games_conf_set_boolean (state->group, window_state_key_name[STATE_KEY_FULLSCREEN], state->is_fullscreen);
   }
 
+  _games_debug_print (GAMES_DEBUG_WINDOW_STATE,
+                      "  > new state: is-maximised:%s is-fullscreen:%s\n",
+                      state->is_maximised ? "t" : "f",
+                      state->is_fullscreen ? "t" : "f");
+
+
   return FALSE;
 }
 
+#ifndef HAVE_HILDON
+
+static char *
+games_conf_get_accel_map_path (GamesConf *conf,
+                               gboolean ensure_dir_exists)
+{
+  GamesConfPrivate *priv = conf->priv;
+  char *game_name, *conf_dir;
+  char *conf_file = NULL;
+
+  game_name = g_ascii_strdown (priv->game_name, -1);
+
+#ifdef HAVE_GNOME
+  conf_dir = g_build_filename (g_get_home_dir (), ".gnome2", "accels", NULL);
+#else
+  conf_dir = g_build_filename (g_get_user_config_dir (), "gnome-games", NULL);
+#endif
+  if (!conf_dir)
+    goto loser;
+
+  /* Mode 0700 per the XDG basedir spec */
+  if (ensure_dir_exists &&
+      g_mkdir_with_parents (conf_dir, 0700) < 0) {
+    int err = errno;
+
+    if (err != EEXIST) {
+      g_warning ("Failed to create config directory \"%s\": %s\n", conf_dir, g_strerror (err));
+      goto loser;
+    }
+  }
+
+#ifdef HAVE_GNOME
+  conf_file = g_build_filename (conf_dir, game_name, NULL);
+#else
+{
+  char *accelmap_filename;
+
+  accelmap_filename = g_strdup_printf ("%s.%s", game_name, ACCELMAP_EXT);
+  conf_file = g_build_filename (conf_dir, accelmap_filename, NULL);
+  g_free (accelmap_filename);
+}
+#endif
+
+loser:
+  g_free (conf_dir);
+  g_free (game_name);
+
+  return conf_file;
+}
+
+static void
+games_conf_load_accel_map (GamesConf *conf)
+{
+  char *conf_file;
+
+  conf_file = games_conf_get_accel_map_path (conf, FALSE);
+  if (!conf_file)
+    return;
+
+  gtk_accel_map_load (conf_file);
+  g_free (conf_file);
+}
+
+static void
+games_conf_save_accel_map (GamesConf *conf)
+{
+  char *conf_file;
+
+  conf_file = games_conf_get_accel_map_path (conf, TRUE);
+  if (!conf_file)
+    return;
+
+  gtk_accel_map_save (conf_file);
+  g_free (conf_file);
+}
+
+#endif /* !HAVE_HILDON */
+
 #ifdef HAVE_GNOME
 
 static void
@@ -258,9 +375,6 @@ games_conf_constructor (GType type,
 #ifndef HAVE_GNOME
   char *conf_file;
   GError *error = NULL;
-#ifndef HAVE_MAEMO
-  char *accelmap_filename;
-#endif /* !HAVE_MAEMO */
 #endif /* HAVE_GNOME */
 
   g_assert (instance == NULL);
@@ -299,8 +413,7 @@ games_conf_constructor (GType type,
   priv->key_file = g_key_file_new ();
   if (!g_key_file_load_from_file (priv->key_file, conf_file, 0, &error)) {
     /* Don't warn on non-existent file */
-    if (error->domain != G_FILE_ERROR ||
-        error->code != G_FILE_ERROR_NOENT) {
+    if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT)) {
       g_warning ("Failed to read settings from \"%s\": %s",
                   conf_file, error->message);
     }
@@ -312,17 +425,12 @@ games_conf_constructor (GType type,
 
   g_free (conf_file);
 
-  /* Load the accel map, which libgnome does for us in the HAVE_GNOME case */
-#ifndef HAVE_MAEMO
-  accelmap_filename = g_strdup_printf ("%s.%s", game_name, ACCELMAP_EXT);
-  conf_file = g_build_filename (g_get_user_config_dir(), "gnome-games", accelmap_filename, NULL);
-  gtk_accel_map_load (conf_file);
-  g_free (accelmap_filename);
-  g_free (conf_file);
-#endif /* !HAVE_MAEMO */
-
 #endif /* HAVE_GNOME */
 
+#ifndef HAVE_HILDON
+  games_conf_load_accel_map (conf);
+#endif /* !HAVE_HILDON */
+
   g_free (game_name);
 
   return object;
@@ -334,10 +442,12 @@ games_conf_finalize (GObject *object)
   GamesConf *conf = GAMES_CONF (object);
   GamesConfPrivate *priv = conf->priv;
 
-#ifdef HAVE_GNOME
+#ifndef HAVE_HILDON
   /* Save the accel map */
-  gnome_accelerators_sync ();
+  games_conf_save_accel_map (conf);
+#endif /* !HAVE_HILDON */
 
+#ifdef HAVE_GNOME
   gconf_client_remove_dir (priv->gconf_client, priv->base_path, NULL);
 
   g_free (priv->base_path);
@@ -349,9 +459,6 @@ games_conf_finalize (GObject *object)
   char *game_name, *conf_file, *conf_dir, *data = NULL;
   gsize len = 0;
   GError *error = NULL;
-#ifndef HAVE_MAEMO
-  char *accelmap_filename;
-#endif /* !HAVE_MAEMO */
 
   game_name = g_ascii_strdown (priv->game_name, -1);
   conf_file = g_build_filename (g_get_user_config_dir (), "gnome-games", game_name, NULL);
@@ -386,16 +493,6 @@ loser:
   g_free (data);
   g_free (conf_file);
   g_free (conf_dir);
-
-  /* Save the accel map */
-#ifndef HAVE_MAEMO
-  accelmap_filename = g_strdup_printf ("%s.%s", game_name, ACCELMAP_EXT);
-  conf_file = g_build_filename (g_get_user_config_dir(), "gnome-games", accelmap_filename, NULL);
-  gtk_accel_map_save (conf_file);
-  g_free (accelmap_filename);
-  g_free (conf_file);
-#endif /* !HAVE_MAEMO */
-
   g_free (game_name);
 
   g_free (priv->main_group);
@@ -423,6 +520,10 @@ games_conf_set_property (GObject *object,
     case PROP_GAME_NAME:
       priv->game_name = g_value_dup_string (value);
       break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
   }
 }
 
@@ -477,6 +578,13 @@ games_conf_initialise (const char *game_name)
                            "game-name", game_name,
                            NULL);
 
+#ifdef HAVE_GNOME
+  /* GConf uses ORBit2 which needs threads (but it's too late to call
+   * g_thread_init() here). See bug #547885.
+   */
+  g_assert (g_thread_supported ());
+#endif
+
   return !instance->priv->need_init;
 }
 
@@ -491,6 +599,7 @@ games_conf_shutdown (void)
   g_assert (instance != NULL);
 
   g_object_unref (instance);
+  instance = NULL;
 }
 
 /**
@@ -878,6 +987,33 @@ games_conf_get_boolean (const char *group, const char *key,
 #endif /* HAVE_GNOME */
 }
 
+/**
+ * games_conf_get_boolean_width_default:
+ * @group: the group name, or %NULL to use the default group
+ * @key: the key name
+ * @def_value: the default value
+ *
+ * Returns the boolean associated with @key in @group, or @def_value if
+ * @key is not set, or an error occurred
+ *
+ * Returns: a boolean
+ */
+gboolean
+games_conf_get_boolean_with_default (const char *group, const char *key,
+                                     gboolean def_value)
+{
+  GError *error = NULL;
+  gboolean value;
+
+  value = games_conf_get_boolean (group, key, &error);
+  if (error) {
+    g_error_free (error);
+    value = def_value;
+  }
+
+  return value;
+}
+
 /**
  * games_conf_set_boolean:
  * @group: the group name, or %NULL to use the default group
@@ -919,9 +1055,8 @@ double
 games_conf_get_double (const char *group, const char *key,
                        GError ** error)
 {
-  GamesConfPrivate *priv = instance->priv;
-
 #if defined(HAVE_GNOME)
+  GamesConfPrivate *priv = instance->priv;
   double value;
   char *key_name;
 
@@ -930,11 +1065,14 @@ games_conf_get_double (const char *group, const char *key,
   g_free (key_name);
 
   return value;
-#elif defined(HAVE_MAEMO)
-#warning games_conf_set_double not supported on maemo!
-  return 0.0;
-#else
+#elif GLIB_CHECK_VERSION (2, 12, 0)
+  GamesConfPrivate *priv = instance->priv;
+
   return g_key_file_get_double (priv->key_file, group ? group : priv->main_group, key, error);
+#else
+#warning games_conf_get_double not implemented on glib < 2.12!
+  /* Not supported */
+  return 0.0;
 #endif /* HAVE_GNOME */
 }
 
@@ -949,19 +1087,20 @@ games_conf_get_double (const char *group, const char *key,
 void
 games_conf_set_double (const char *group, const char *key, double value)
 {
-  GamesConfPrivate *priv = instance->priv;
-
 #if defined(HAVE_GNOME)
+  GamesConfPrivate *priv = instance->priv;
   char *key_name;
 
   key_name = get_gconf_key_name (group, key);
   gconf_client_set_float (priv->gconf_client, key_name, value, NULL);
   g_free (key_name);
-#elif defined(HAVE_MAEMO)
-#warning games_conf_set_double not implemented on maemo!
-#else
+#elif GLIB_CHECK_VERSION (2, 12, 0)
+  GamesConfPrivate *priv = instance->priv;
+
   g_key_file_set_double (priv->key_file, group ? group : priv->main_group, key, value);
   g_signal_emit (instance, signals[VALUE_CHANGED], 0, group, key);
+#else
+#warning games_conf_set_double not implemented on glib < 2.12!
 #endif /* HAVE_GNOME */
 }
 
@@ -1121,7 +1260,13 @@ games_conf_add_window (GtkWindow *window,
   g_return_if_fail (GTK_IS_WINDOW (window));
   g_return_if_fail (!GTK_WIDGET_REALIZED (window));
 
+#if GLIB_CHECK_VERSION (2, 10, 0)
   state = g_slice_new0 (WindowState);
+#else
+  state = g_new0 (WindowState, 1);
+#endif
+
+  state->window = window;
   state->group = g_strdup (group);
   g_object_set_data_full (G_OBJECT (window), "GamesConf::WindowState",
                           state, (GDestroyNotify) free_window_state);
@@ -1131,18 +1276,28 @@ games_conf_add_window (GtkWindow *window,
   g_signal_connect (window, "window-state-event",
                     G_CALLBACK (window_state_event_cb), state);
 
-  maximised = games_conf_get_boolean (NULL, window_state_key_name[STATE_KEY_MAXIMISED], NULL);
-  fullscreen = games_conf_get_boolean (NULL, window_state_key_name[STATE_KEY_FULLSCREEN], NULL);
-  width = games_conf_get_integer (NULL, window_state_key_name[STATE_KEY_WIDTH], NULL);
-  height = games_conf_get_integer (NULL, window_state_key_name[STATE_KEY_HEIGHT], NULL);
+  maximised = games_conf_get_boolean (group, window_state_key_name[STATE_KEY_MAXIMISED], NULL);
+  fullscreen = games_conf_get_boolean (group, window_state_key_name[STATE_KEY_FULLSCREEN], NULL);
+  width = games_conf_get_integer (group, window_state_key_name[STATE_KEY_WIDTH], NULL);
+  height = games_conf_get_integer (group, window_state_key_name[STATE_KEY_HEIGHT], NULL);
 
   if (width > 0 && height > 0) {
+    _games_debug_print (GAMES_DEBUG_WINDOW_STATE,
+                        "[window %p] restoring size %dx%d\n",
+                        state->window,
+                        width, height);
     gtk_window_set_default_size (GTK_WINDOW (window), width, height);
   }
   if (maximised) {
+    _games_debug_print (GAMES_DEBUG_WINDOW_STATE,
+                        "[window %p] restoring maximised state\n",
+                        state->window);
     gtk_window_maximize (GTK_WINDOW (window));
   }
   if (fullscreen) {
+    _games_debug_print (GAMES_DEBUG_WINDOW_STATE,
+                        "[window %p] restoring fullscreen state\n",
+                        state->window);
     gtk_window_fullscreen (GTK_WINDOW (window));
   }
 }
