@@ -22,10 +22,13 @@
 /* A lot of this was written by following the sorce for GtkFrame and
  * GtkAspectFrame. */
 
+#include <config.h>
+
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
 #include "games-gridframe.h"
+#include "games-gtk-compat.h"
 
 enum {
   PROP_0,
@@ -37,6 +40,8 @@ enum {
   PROP_Y_ALIGN
 };
 
+G_DEFINE_TYPE (GamesGridFrame, games_grid_frame,GTK_TYPE_BIN)
+
 void
 games_grid_frame_set (GamesGridFrame * frame, gint newxmult, gint newymult)
 {
@@ -150,7 +155,7 @@ static void
 games_grid_frame_size_request (GtkWidget * widget,
 			       GtkRequisition * requisition)
 {
-  GtkWidget *child = GTK_BIN (widget)->child;
+  GtkWidget *child = gtk_bin_get_child (GTK_BIN (widget));
 
   requisition->width = 1;
   requisition->height = 1;
@@ -165,11 +170,11 @@ games_grid_frame_size_allocate (GtkWidget * widget,
 				GtkAllocation * allocation)
 {
   GamesGridFrame *frame = GAMES_GRID_FRAME (widget);
-  GtkWidget *child = GTK_BIN (widget)->child;
+  GtkWidget *child = gtk_bin_get_child (GTK_BIN (widget));
   GtkAllocation child_allocation;
   gint xsize, ysize, size;
 
-  widget->allocation = *allocation;
+  gtk_widget_set_allocation (widget, allocation);
 
   xsize = MAX (1, (allocation->width - frame->xpadding) / frame->xmult);
   ysize = MAX (1, (allocation->height - frame->ypadding) / frame->ymult);
@@ -191,7 +196,7 @@ games_grid_frame_size_allocate (GtkWidget * widget,
        child_allocation.y != frame->old_allocation.y ||
        child_allocation.width != frame->old_allocation.width ||
        child_allocation.height != frame->old_allocation.height))
-    gdk_window_invalidate_rect (widget->window, &widget->allocation, FALSE);
+    gdk_window_invalidate_rect (gtk_widget_get_window (widget), allocation, FALSE);
 
   if (child && GTK_WIDGET_VISIBLE (child))
     gtk_widget_size_allocate (child, &child_allocation);
@@ -272,30 +277,6 @@ games_grid_frame_init (GamesGridFrame * frame)
   frame->yalign = 0.5;
 }
 
-GType
-games_grid_frame_get_type (void)
-{
-  static GType type = 0;
-
-  if (!type) {
-    static const GTypeInfo info = {
-      sizeof (GamesGridFrameClass),
-      NULL,
-      NULL,
-      (GClassInitFunc) games_grid_frame_class_init,
-      NULL,
-      NULL,
-      sizeof (GamesGridFrame),
-      0,
-      (GInstanceInitFunc) games_grid_frame_init,
-    };
-
-    type = g_type_register_static (GTK_TYPE_BIN, "GamesGridFrame", &info, 0);
-  }
-
-  return type;
-}
-
 GtkWidget *
 games_grid_frame_new (gint width, gint height)
 {
