@@ -14,6 +14,7 @@
 #include <gtk/gtk.h>
 
 #include <libgames-support/games-conf.h>
+#include <libgames-support/games-runtime.h>
 
 #include "same-gnome.h"
 
@@ -61,6 +62,8 @@ gboolean draw_cursor = FALSE;
 
 gboolean fast_animation = FALSE;
 
+static gint animation_timer_id = 0;
+
 void
 pixels_to_logical (gint px, gint py, gint * lx, gint * ly)
 {
@@ -81,7 +84,7 @@ redraw (void)
  * This is the natural way given the directions the balls fall. */
 static void
 draw_ball_with_offset (GtkWidget * canvas, game_cell * p, int x,
-		       int y, int dx, int dy)
+                       int y, int dx, int dy)
 {
   int colour, frame;
   GdkPixmap *pixmap;
@@ -93,30 +96,32 @@ draw_ball_with_offset (GtkWidget * canvas, game_cell * p, int x,
     /* We don't add an offset to the blank tile since it is part of 
      * the background. The background movement is compensated for when
      * the other tiles were created. */
-    gdk_draw_drawable (canvas->window, canvas->style->black_gc,
-		       blank_pixmap, 0, 0, x * tile_size,
-		       y * tile_size, tile_size, tile_size);
+    gdk_draw_drawable (gtk_widget_get_window (canvas),
+                       gtk_widget_get_style (canvas)->black_gc,
+                       blank_pixmap, 0, 0, x * tile_size,
+                       y * tile_size, tile_size, tile_size);
   } else {
     pixmap = pixmaps[colour][frame];
-    gdk_draw_drawable (canvas->window, canvas->style->black_gc,
-		       pixmap, 0, 0, x * tile_size + dx,
-		       y * tile_size - dy, tile_size, tile_size);
+    gdk_draw_drawable (gtk_widget_get_window (canvas),
+                       gtk_widget_get_style (canvas)->black_gc,
+                       pixmap, 0, 0, x * tile_size + dx,
+                       y * tile_size - dy, tile_size, tile_size);
   }
 
   /* Draw the cursor is needed. */
   if (draw_cursor && (x == cursor_x) && (y == cursor_y)) {
     if (cursorgc == NULL) {
-      cursorgc = gdk_gc_new (canvaswidget->window);
+      cursorgc = gdk_gc_new (gtk_widget_get_window (canvaswidget));
       gdk_colormap_alloc_color (gdk_colormap_get_system (),
-				&cursorcolor, TRUE, TRUE);
+                                &cursorcolor, TRUE, TRUE);
       gdk_gc_set_foreground (cursorgc, &cursorcolor);
       gdk_gc_set_line_attributes (cursorgc, 2, GDK_LINE_SOLID,
-				  GDK_CAP_BUTT, GDK_JOIN_MITER);
+                                  GDK_CAP_BUTT, GDK_JOIN_MITER);
     }
 
-    gdk_draw_rectangle (canvaswidget->window, cursorgc,
-			FALSE, x * tile_size + 2, y * tile_size + 2,
-			tile_size - 3, tile_size - 3);
+    gdk_draw_rectangle (gtk_widget_get_window (canvaswidget), cursorgc,
+                        FALSE, x * tile_size + 2, y * tile_size + 2,
+                        tile_size - 3, tile_size - 3);
   }
 
 }
@@ -140,7 +145,7 @@ cursor_erase (void)
 {
   draw_cursor = FALSE;
   draw_ball (canvaswidget, get_game_cell (cursor_x, cursor_y),
-	     cursor_x, cursor_y);
+             cursor_x, cursor_y);
 }
 
 void
@@ -160,14 +165,14 @@ expose_cb (GtkWidget * canvas, GdkEventExpose * event, gpointer data)
   game_cell dummy = { -1, 0, 0, 0 };
 
   if (gridgc == NULL) {
-    gridgc = gdk_gc_new (canvas->window);
+    gridgc = gdk_gc_new (gtk_widget_get_window (canvas));
     gdk_colormap_alloc_color (gdk_colormap_get_system (),
-			      &gridcolor, TRUE, TRUE);
+                              &gridcolor, TRUE, TRUE);
     gdk_gc_set_foreground (gridgc, &gridcolor);
 
-    bggc = gdk_gc_new (canvas->window);
+    bggc = gdk_gc_new (gtk_widget_get_window (canvas));
     gdk_colormap_alloc_color (gdk_colormap_get_system (),
-			      &bgcolor, TRUE, TRUE);
+                              &bgcolor, TRUE, TRUE);
     gdk_gc_set_foreground (bggc, &bgcolor);
   }
 
@@ -180,50 +185,50 @@ expose_cb (GtkWidget * canvas, GdkEventExpose * event, gpointer data)
     p = board;
     for (y = 0; y < board_height; y++) {
       for (x = 0; x < board_width; x++) {
-	/* Only draw stationary frames. Fill the rest in as blanks 
-	 * and let the animation sort it out. */
-	if (p->frame == 0)
-	  draw_ball (canvas, p, x, y);
-	else
-	  draw_ball (canvas, &dummy, x, y);
-	p++;
+        /* Only draw stationary frames. Fill the rest in as blanks 
+         * and let the animation sort it out. */
+        if (p->frame == 0)
+          draw_ball (canvas, p, x, y);
+        else
+          draw_ball (canvas, &dummy, x, y);
+        p++;
       }
     }
 
     /* Fixup the left and bottom lines. */
     if ((event->area.y + event->area.height + 1) >= board_height * tile_size) {
-      gdk_draw_line (canvas->window, gridgc,
-		     event->area.x, board_height * tile_size,
-		     event->area.x + event->area.width - 2,
-		     board_height * tile_size);
+      gdk_draw_line (gtk_widget_get_window (canvas), gridgc,
+                     event->area.x, board_height * tile_size,
+                     event->area.x + event->area.width - 2,
+                     board_height * tile_size);
     }
     if ((event->area.x + event->area.width + 1) >= board_width * tile_size) {
-      gdk_draw_line (canvas->window, gridgc,
-		     board_width * tile_size, event->area.y,
-		     board_width * tile_size,
-		     event->area.y + event->area.height - 2);
+      gdk_draw_line (gtk_widget_get_window (canvas), gridgc,
+                     board_width * tile_size, event->area.y,
+                     board_width * tile_size,
+                     event->area.y + event->area.height - 2);
     }
 
-  } else {			/* Draw only the grid. */
+  } else {                        /* Draw only the grid. */
 
-    gdk_draw_rectangle (canvas->window, bggc, TRUE,
-			event->area.x, event->area.y,
-			event->area.width, event->area.height);
+    gdk_draw_rectangle (gtk_widget_get_window (canvas), bggc, TRUE,
+                        event->area.x, event->area.y,
+                        event->area.width, event->area.height);
 
     /* Vertical lines. */
     for (x = tile_size * (event->area.x / tile_size);
-	 x <= tile_size * ((event->area.x + event->area.width) / tile_size);
-	 x += tile_size) {
-      gdk_draw_line (canvas->window, gridgc, x,
-		     event->area.y, x, event->area.y + event->area.height);
+         x <= tile_size * ((event->area.x + event->area.width) / tile_size);
+         x += tile_size) {
+      gdk_draw_line (gtk_widget_get_window (canvas), gridgc, x,
+                     event->area.y, x, event->area.y + event->area.height);
     }
 
     /* Horizontal lines. */
     for (x = tile_size * (event->area.y / tile_size);
-	 x <= tile_size * ((event->area.y + event->area.height) / tile_size);
-	 x += tile_size) {
-      gdk_draw_line (canvas->window, gridgc,
-		     event->area.x, x, event->area.x + event->area.width, x);
+         x <= tile_size * ((event->area.y + event->area.height) / tile_size);
+         x += tile_size) {
+      gdk_draw_line (gtk_widget_get_window (canvas), gridgc,
+                     event->area.x, x, event->area.x + event->area.width, x);
     }
 
   }
@@ -262,7 +267,7 @@ render_cb (GtkWidget * canvas)
   GtkWidget *dialog;
   static guchar r, g, b, br, bg, bb;
   guchar tr, tg, tb;
-
+    
   if (idle_state == INIT) {
     last_tile_size = tile_size;
     ftile_size = tile_size - 1;
@@ -276,7 +281,7 @@ render_cb (GtkWidget * canvas)
 
     /* Draw up the background with the top and left grid lines. */
     bg_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, tile_size,
-				tile_size);
+                                tile_size);
     /* FIXME: This is too large for this function, ship off else-where. */
     p = gdk_pixbuf_get_pixels (bg_pixbuf);
     l = gdk_pixbuf_get_rowstride (bg_pixbuf);
@@ -297,17 +302,18 @@ render_cb (GtkWidget * canvas)
       *p++ = g;
       *p++ = b;
       for (i = 1; i < tile_size; i++) {
-	*p++ = br;
-	*p++ = bg;
-	*p++ = bb;
+        *p++ = br;
+        *p++ = bg;
+        *p++ = bb;
       }
     }
     if (blank_pixmap != NULL)
       g_object_unref (blank_pixmap);
-    blank_pixmap = gdk_pixmap_new (canvas->window, tile_size, tile_size, -1);
-    gdk_draw_pixbuf (blank_pixmap, canvas->style->black_gc, bg_pixbuf,
-		     0, 0, 0, 0, tile_size, tile_size, GDK_RGB_DITHER_NORMAL,
-		     0, 0);
+    blank_pixmap = gdk_pixmap_new (gtk_widget_get_window (canvas), tile_size,
+                                   tile_size, -1);
+    gdk_draw_pixbuf (blank_pixmap, gtk_widget_get_style (canvas)->black_gc,
+                     bg_pixbuf, 0, 0, 0, 0, tile_size, tile_size,
+                     GDK_RGB_DITHER_NORMAL, 0, 0);
 
     /* This code is far from perfect. For starters it only checks to
      * see if the file exists before deciding that the filename is the
@@ -319,36 +325,36 @@ render_cb (GtkWidget * canvas)
     if (!g_file_test (filename, G_FILE_TEST_EXISTS)) {
       g_free (filename);
       /* Then look in the system directory. */
-      filename = g_build_filename (THEMEDIR, theme, NULL);
+      filename = g_build_filename (games_runtime_get_directory (GAMES_RUNTIME_GAME_THEME_DIRECTORY), THEME_VERSION, theme, NULL);
       if (!g_file_test (filename, G_FILE_TEST_EXISTS)) {
-	/* Some old themes had a -sync just before the suffix. This was
-	 * hidden from the user and has been eliminated from the names
-	 * of reworked themes. Check in case we had one of this form. */
-	length = strlen (filename);
-	substring = g_strstr_len (filename, length, "-sync");
-	suffix = g_strrstr (filename, ".png");
-	if (substring && suffix) {
-	  g_stpcpy (substring, suffix);
-	}
-	if (!g_file_test (filename, G_FILE_TEST_EXISTS)) {
-	  /* We have changed some themes from .png to .svg, try that. */
-	  suffix = g_strrstr (filename, ".png");
-	  if (suffix) {
-	    g_stpcpy (suffix, ".svg");
-	  }
-	  if (!g_file_test (filename, G_FILE_TEST_EXISTS)) {
-	    /* And finally fall back to the default. */
-	    g_free (filename);
-	    filename = g_build_filename (THEMEDIR, DEFAULT_THEME, NULL);
-	  }
-	}
+        /* Some old themes had a -sync just before the suffix. This was
+         * hidden from the user and has been eliminated from the names
+         * of reworked themes. Check in case we had one of this form. */
+        length = strlen (filename);
+        substring = g_strstr_len (filename, length, "-sync");
+        suffix = g_strrstr (filename, ".png");
+        if (substring && suffix) {
+          g_stpcpy (substring, suffix);
+        }
+        if (!g_file_test (filename, G_FILE_TEST_EXISTS)) {
+          /* We have changed some themes from .png to .svg, try that. */
+          suffix = g_strrstr (filename, ".png");
+          if (suffix) {
+            g_stpcpy (suffix, ".svg");
+          }
+          if (!g_file_test (filename, G_FILE_TEST_EXISTS)) {
+            /* And finally fall back to the default. */
+            g_free (filename);
+            filename = g_build_filename (games_runtime_get_directory (GAMES_RUNTIME_GAME_THEME_DIRECTORY), THEME_VERSION, DEFAULT_THEME, NULL);
+          }
+        }
       }
     }
 
     file_pixbuf = gdk_pixbuf_new_from_file_at_size (filename,
-						    ftile_size * NFRAMESSPIN,
-						    ftile_size * MAX_COLOURS,
-						    NULL);
+                                                    ftile_size * NFRAMESSPIN,
+                                                    ftile_size * MAX_COLOURS,
+                                                    NULL);
     g_free (filename);
 
     idle_state = DRAW;
@@ -357,16 +363,16 @@ render_cb (GtkWidget * canvas)
     if (file_pixbuf == NULL) {
       /* FIXME: We haven't got the parent window right. */
       dialog = gtk_message_dialog_new (NULL,
-				       GTK_DIALOG_MODAL |
-				       GTK_DIALOG_DESTROY_WITH_PARENT,
-				       GTK_MESSAGE_ERROR,
-				       GTK_BUTTONS_NONE,
-				       _("No theme data was found."));
+                                       GTK_DIALOG_MODAL |
+                                       GTK_DIALOG_DESTROY_WITH_PARENT,
+                                       GTK_MESSAGE_ERROR,
+                                       GTK_BUTTONS_NONE,
+                                       _("No theme data was found."));
       gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_QUIT,
-			     GTK_RESPONSE_OK);
+                             GTK_RESPONSE_OK);
       gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						_
-						("It is impossible to play the game. Please check that the game has been installed correctly and try again."));
+                                                _
+                                                ("It is impossible to play the game. Please check that the game has been installed correctly and try again."));
       gtk_dialog_run (GTK_DIALOG (dialog));
       gtk_main_quit ();
       return FALSE;
@@ -376,31 +382,31 @@ render_cb (GtkWidget * canvas)
       idle_state = INIT;
     } else {
       if (pixmaps[n][m] != NULL)
-	g_object_unref (pixmaps[n][m]);
+        g_object_unref (pixmaps[n][m]);
 
       tile = gdk_pixbuf_copy (bg_pixbuf);
       gdk_pixbuf_composite (file_pixbuf, tile, 1, 1, ftile_size,
-			    ftile_size, 1.0 - m * ftile_size,
-			    1.0 - n * ftile_size,
-			    1.0, 1.0, GDK_INTERP_TILES, 255);
+                            ftile_size, 1.0 - m * ftile_size,
+                            1.0 - n * ftile_size,
+                            1.0, 1.0, GDK_INTERP_TILES, 255);
 
-      pixmaps[n][m] = gdk_pixmap_new (canvas->window, tile_size, tile_size,
-				      -1);
-      gdk_draw_pixbuf (pixmaps[n][m], canvas->style->black_gc, tile,
-		       0, 0, 0, 0, tile_size, tile_size,
-		       GDK_RGB_DITHER_NORMAL, 0, 0);
+      pixmaps[n][m] = gdk_pixmap_new (gtk_widget_get_window (canvas), tile_size,
+                                      tile_size, -1);
+      gdk_draw_pixbuf (pixmaps[n][m], gtk_widget_get_style (canvas)->black_gc,
+                       tile, 0, 0, 0, 0, tile_size, tile_size,
+                       GDK_RGB_DITHER_NORMAL, 0, 0);
 
       g_object_unref (tile);
 
       m++;
       if (m == NFRAMESSPIN) {
-	m = 0;
-	n++;
-	if (n == MAX_COLOURS) {
-	  idle_state = DEST;
-	  n = 0;
-	  m = DESTFRAMESOFS;
-	}
+        m = 0;
+        n++;
+        if (n == MAX_COLOURS) {
+          idle_state = DEST;
+          n = 0;
+          m = DESTFRAMESOFS;
+        }
       }
     }
   } else if (idle_state == DEST) {
@@ -408,7 +414,7 @@ render_cb (GtkWidget * canvas)
       idle_state = INIT;
     } else {
       if (pixmaps[n][m] != NULL)
-	g_object_unref (pixmaps[n][m]);
+        g_object_unref (pixmaps[n][m]);
 
       tile = gdk_pixbuf_copy (bg_pixbuf);
 
@@ -426,31 +432,31 @@ render_cb (GtkWidget * canvas)
       tg = bg + ((0xff - bg) >> shift);
       tb = bb + ((0xff - bb) >> shift);
       for (j = 1; j < tile_size; j++) {
-	p = bp + j * l + 3;
-	for (i = 1; i < tile_size; i++) {
-	  *p++ = tr;
-	  *p++ = tg;
-	  *p++ = tb;
-	}
+        p = bp + j * l + 3;
+        for (i = 1; i < tile_size; i++) {
+          *p++ = tr;
+          *p++ = tg;
+          *p++ = tb;
+        }
       }
 
-      pixmaps[n][m] = gdk_pixmap_new (canvas->window, tile_size, tile_size,
-				      -1);
-      gdk_draw_pixbuf (pixmaps[n][m], canvas->style->black_gc, tile,
-		       0, 0, 0, 0, tile_size, tile_size,
-		       GDK_RGB_DITHER_NORMAL, 0, 0);
+      pixmaps[n][m] = gdk_pixmap_new (gtk_widget_get_window (canvas), tile_size,
+                                      tile_size, -1);
+      gdk_draw_pixbuf (pixmaps[n][m], gtk_widget_get_style (canvas)->black_gc,
+                       tile, 0, 0, 0, 0, tile_size, tile_size,
+                       GDK_RGB_DITHER_NORMAL, 0, 0);
 
       g_object_unref (tile);
 
       m++;
       if (m == MOVEDFRAMESOFS) {
-	m = DESTFRAMESOFS;
-	n++;
-	if (n == MAX_COLOURS) {
-	  idle_state = MOVED;
-	  n = 0;
-	  m = MOVEDFRAMESOFS;
-	}
+        m = DESTFRAMESOFS;
+        n++;
+        if (n == MAX_COLOURS) {
+          idle_state = MOVED;
+          n = 0;
+          m = MOVEDFRAMESOFS;
+        }
       }
     }
   } else if (idle_state == MOVED) {
@@ -458,7 +464,7 @@ render_cb (GtkWidget * canvas)
       idle_state = INIT;
     } else {
       if (pixmaps[n][m] != NULL)
-	g_object_unref (pixmaps[n][m]);
+        g_object_unref (pixmaps[n][m]);
 
       x = m - MOVEDFRAMESOFS;
       tile = gdk_pixbuf_copy (bg_pixbuf);
@@ -470,41 +476,41 @@ render_cb (GtkWidget * canvas)
       p = bp + 3;
       /* Remove the horizontal line... */
       for (i = 1; i < tile_size; i++) {
-	*p++ = br;
-	*p++ = bg;
-	*p++ = bb;
+        *p++ = br;
+        *p++ = bg;
+        *p++ = bb;
       }
       j = tile_size - 1 - ((x + 1) * (tile_size - 1)) / (NFRAMESMOVED);
       downoffsets[x] = j;
       p = bp + j * l + 3;
       /* ...and redraw it in the correct place. */
       for (i = 1; i < tile_size; i++) {
-	*p++ = r;
-	*p++ = g;
-	*p++ = b;
+        *p++ = r;
+        *p++ = g;
+        *p++ = b;
       }
 
       gdk_pixbuf_composite (file_pixbuf, tile, 1, 1, ftile_size,
-			    ftile_size, 1.0, 1.0 - n * ftile_size,
-			    1.0, 1.0, GDK_INTERP_TILES, 255);
+                            ftile_size, 1.0, 1.0 - n * ftile_size,
+                            1.0, 1.0, GDK_INTERP_TILES, 255);
 
-      pixmaps[n][m] = gdk_pixmap_new (canvas->window, tile_size, tile_size,
-				      -1);
-      gdk_draw_pixbuf (pixmaps[n][m], canvas->style->black_gc, tile,
-		       0, 0, 0, 0, tile_size, tile_size,
-		       GDK_RGB_DITHER_NORMAL, 0, 0);
+      pixmaps[n][m] = gdk_pixmap_new (gtk_widget_get_window (canvas), tile_size,
+                                      tile_size, -1);
+      gdk_draw_pixbuf (pixmaps[n][m], gtk_widget_get_style (canvas)->black_gc,
+                       tile, 0, 0, 0, 0, tile_size, tile_size,
+                       GDK_RGB_DITHER_NORMAL, 0, 0);
 
       g_object_unref (tile);
 
       m++;
       if (m == MOVELFRAMESOFS) {
-	m = MOVEDFRAMESOFS;
-	n++;
-	if (n == MAX_COLOURS) {
-	  idle_state = MOVEL;
-	  n = 0;
-	  m = MOVELFRAMESOFS;
-	}
+        m = MOVEDFRAMESOFS;
+        n++;
+        if (n == MAX_COLOURS) {
+          idle_state = MOVEL;
+          n = 0;
+          m = MOVELFRAMESOFS;
+        }
       }
     }
   } else if (idle_state == MOVEL) {
@@ -512,7 +518,7 @@ render_cb (GtkWidget * canvas)
       idle_state = INIT;
     } else {
       if (pixmaps[n][m] != NULL)
-	g_object_unref (pixmaps[n][m]);
+        g_object_unref (pixmaps[n][m]);
 
       x = m - MOVELFRAMESOFS;
       tile = gdk_pixbuf_copy (bg_pixbuf);
@@ -524,42 +530,42 @@ render_cb (GtkWidget * canvas)
       p = bp + l;
       /* Remove the vertical line... */
       for (i = 1; i < tile_size; i++) {
-	*p++ = br;
-	*p++ = bg;
-	*p = bb;
-	p += l - 2;
+        *p++ = br;
+        *p++ = bg;
+        *p = bb;
+        p += l - 2;
       }
       j = ((x + 1) * (tile_size - 1)) / (NFRAMESMOVED);
       leftoffsets[x] = tile_size - j;
       p = bp + l + 3 * j;
       /* ...and redraw it in the correct place. */
       for (i = 1; i < tile_size; i++) {
-	*p++ = r;
-	*p++ = g;
-	*p = b;
-	p += l - 2;
+        *p++ = r;
+        *p++ = g;
+        *p = b;
+        p += l - 2;
       }
 
       gdk_pixbuf_composite (file_pixbuf, tile, 1, 1, ftile_size,
-			    ftile_size, 1.0, 1.0 - n * ftile_size,
-			    1.0, 1.0, GDK_INTERP_TILES, 255);
+                            ftile_size, 1.0, 1.0 - n * ftile_size,
+                            1.0, 1.0, GDK_INTERP_TILES, 255);
 
-      pixmaps[n][m] = gdk_pixmap_new (canvas->window, tile_size, tile_size,
-				      -1);
-      gdk_draw_pixbuf (pixmaps[n][m], canvas->style->black_gc, tile,
-		       0, 0, 0, 0, tile_size, tile_size,
-		       GDK_RGB_DITHER_NORMAL, 0, 0);
+      pixmaps[n][m] = gdk_pixmap_new (gtk_widget_get_window (canvas), tile_size,
+                                      tile_size, -1);
+      gdk_draw_pixbuf (pixmaps[n][m], gtk_widget_get_style (canvas)->black_gc,
+                       tile, 0, 0, 0, 0, tile_size, tile_size,
+                       GDK_RGB_DITHER_NORMAL, 0, 0);
 
       g_object_unref (tile);
 
       m++;
       if (m == NFRAMES) {
-	m = MOVELFRAMESOFS;
-	n++;
-	if (n == MAX_COLOURS) {
-	  idle_state = DONE;
-	  pixmaps_ready = TRUE;
-	}
+        m = MOVELFRAMESOFS;
+        n++;
+        if (n == MAX_COLOURS) {
+          idle_state = DONE;
+          pixmaps_ready = TRUE;
+        }
       }
     }
   } else if (idle_state == DONE) {
@@ -571,7 +577,7 @@ render_cb (GtkWidget * canvas)
     idle_id = 0;
     redraw ();
 
-    if (pixmaps_ready)		/* Just in case this was reset after the last idle call. */
+    if (pixmaps_ready)                /* Just in case this was reset after the last idle call. */
       return FALSE;
   }
   return TRUE;
@@ -609,6 +615,7 @@ animation_timer (void)
   int i, j;
   game_cell *p;
   gboolean changestatep;
+  gboolean animating = FALSE;
 
   if (!pixmaps_ready)
     return TRUE;
@@ -622,80 +629,84 @@ animation_timer (void)
     for (i = board_width - 1; i >= 0; i--) {
       switch (p->style) {
       case ANI_STILL:
-	if (p->frame != 0)
-	  p->style = ANI_SPINBACK;
-	break;
+        if (p->frame != 0)
+          p->style = ANI_SPINBACK;
+        break;
       case ANI_REDRAW:
-	draw_ball (canvaswidget, p, i, j);
-	p->style = ANI_STILL;
-	break;
+        draw_ball (canvaswidget, p, i, j);
+        p->style = ANI_STILL;
+        break;
       case ANI_SPIN:
-	p->frame = (p->frame + 1) % NFRAMESSPIN;
-	draw_ball (canvaswidget, p, i, j);
-	break;
+        p->frame = (p->frame + 1) % NFRAMESSPIN;
+        draw_ball (canvaswidget, p, i, j);
+        break;
       case ANI_SPINBACK:
-	if (p->frame > 4)
-	  p->frame = 4;
-	if (p->frame == 0)
-	  p->style = ANI_STILL;
-	else
-	  p->frame--;
-	draw_ball (canvaswidget, p, i, j);
-	break;
+        if (p->frame > 4)
+          p->frame = 4;
+        if (p->frame == 0)
+          p->style = ANI_STILL;
+        else
+          p->frame--;
+        draw_ball (canvaswidget, p, i, j);
+        break;
       case ANI_DESTROY:
-	if (fast_animation)
-	  p->frame = MOVEDFRAMESOFS;
-	else
-	  p->frame++;
-	if (p->frame < DESTFRAMESOFS) {
-	  p->frame = DESTFRAMESOFS;
-	} else if (p->frame < MOVEDFRAMESOFS) {
-
-	} else {
-	  p->colour = NONE;
-	  changestatep = TRUE;
-	  p->frame = 0;
-	  p->style = ANI_STILL;
-	}
-	draw_ball (canvaswidget, p, i, j);
-	break;
+        if (fast_animation)
+           p->frame = MOVEDFRAMESOFS;
+        else
+           p->frame++;
+        if (p->frame < DESTFRAMESOFS) {
+          p->frame = DESTFRAMESOFS;
+        } else if (p->frame < MOVEDFRAMESOFS) {
+              
+        } else {
+          p->colour = NONE;
+          changestatep = TRUE;
+          p->frame = 0;
+          p->style = ANI_STILL;
+        }
+        draw_ball (canvaswidget, p, i, j);
+        break;
       case ANI_MOVE_DOWN:
-	if (game_state == GAME_MOVING_DOWN) {
-	  if (fast_animation)
-	    /* This allows a fast, but not instant, animation. */
-	    p->frame = MOVELFRAMESOFS;
-	  else
-	    p->frame += speed;
-	  if (p->frame >= MOVELFRAMESOFS) {
-	    changestatep = TRUE;
-	    p->frame = 0;
-	    p->style = ANI_STILL;
-	    draw_ball (canvaswidget, p, i, j);
-	  } else {
-	    draw_ball_with_offset (canvaswidget, p, i, j, 0,
-				   downoffsets[p->frame - MOVEDFRAMESOFS]);
-	  }
-	}
-	break;
+        if (game_state == GAME_MOVING_DOWN) {
+          if (fast_animation)
+            /* This allows a fast, but not instant, animation. */
+            p->frame = MOVELFRAMESOFS;
+          else
+            p->frame += speed;
+          if (p->frame >= MOVELFRAMESOFS) {
+            changestatep = TRUE;
+            p->frame = 0;
+            p->style = ANI_STILL;
+            draw_ball (canvaswidget, p, i, j);
+          } else {
+            draw_ball_with_offset (canvaswidget, p, i, j, 0,
+                                   downoffsets[p->frame - MOVEDFRAMESOFS]);
+          }
+        }
+        break;
       case ANI_MOVE_LEFT:
-	if (game_state == GAME_MOVING_LEFT) {
-	  if (fast_animation)
-	    /* This allows a fast, but not instant, animation. */
-	    p->frame = NFRAMES;
-	  else
-	    p->frame += speed;
-	  if (p->frame >= NFRAMES) {
-	    changestatep = TRUE;
-	    p->frame = 0;
-	    p->style = ANI_STILL;
-	    draw_ball (canvaswidget, p, i, j);
-	  } else {
-	    draw_ball_with_offset (canvaswidget, p, i, j,
-				   leftoffsets[p->frame - MOVELFRAMESOFS], 0);
-	  }
-	}
-	break;
+        if (game_state == GAME_MOVING_LEFT) {
+          if (fast_animation)
+            /* This allows a fast, but not instant, animation. */
+            p->frame = NFRAMES;
+          else
+            p->frame += speed;
+            if (p->frame >= NFRAMES) {
+              changestatep = TRUE;
+              p->frame = 0;
+              p->style = ANI_STILL;
+              draw_ball (canvaswidget, p, i, j);
+            } else {
+              draw_ball_with_offset (canvaswidget, p, i, j,
+                                     leftoffsets[p->frame - MOVELFRAMESOFS], 0);
+            }
+        }
+        break;
       }
+
+      if(p->style != ANI_STILL)
+        animating = TRUE;
+
       p--;
     }
   }
@@ -708,6 +719,7 @@ animation_timer (void)
    * game_state. However we only want to do these things once, so
    * we flag them and check them down here once the loop has finished. */
   if (changestatep) {
+    animating = TRUE;
     /* FIXME: Is this really the place to change the game state ?
      * We do need to signal the end of the animation though so maybe ...
      * or maybe we move it over to game.c 
@@ -716,14 +728,14 @@ animation_timer (void)
     switch (game_state) {
     case GAME_DESTROYING:
       if (mark_falling_balls () == 0)
-	if (mark_shifting_balls ()) {
-	  game_state = GAME_MOVING_LEFT;
-	  speed = 1.0;
-	} else {
-	  end_of_move ();
+        if (mark_shifting_balls ()) {
+          game_state = GAME_MOVING_LEFT;
+          speed = 1.0;
+        } else {
+          end_of_move ();
       } else {
-	game_state = GAME_MOVING_DOWN;
-	speed = 1.0;
+        game_state = GAME_MOVING_DOWN;
+        speed = 1.0;
       }
       break;
       /* FIXME: Because of the way we do the drawing (with the offset)
@@ -731,28 +743,40 @@ animation_timer (void)
        * flicker. */
     case GAME_MOVING_DOWN:
       if (mark_falling_balls () == 0) {
-	if (mark_shifting_balls ()) {
-	  game_state = GAME_MOVING_LEFT;
-	} else {
-	  end_of_move ();
-	  select_cells ();
-	}
-	speed = 1.0;
+        if (mark_shifting_balls ()) {
+          game_state = GAME_MOVING_LEFT;
+        } else {
+          end_of_move ();
+          select_cells ();
+        }
+        speed = 1.0;
       } else {
 
       }
       break;
     case GAME_MOVING_LEFT:
       if (!mark_shifting_balls ()) {
-	end_of_move ();
-	select_cells ();
+        end_of_move ();
+        select_cells ();
       }
       break;
     }
   }
 
+  if (animating) {
+    return TRUE;
+  } else {
+    animation_timer_id = 0;
+    return FALSE;
+  }
+}
 
-  return TRUE;
+void
+start_animation (void)
+{
+  /* 16 frames/second. */
+  if (animation_timer_id == 0)
+    animation_timer_id = g_timeout_add (62, (GSourceFunc) animation_timer, NULL);    
 }
 
 void
@@ -764,7 +788,7 @@ resize_graphics (void)
   if (first_run) {
     for (n = 0; n < MAX_COLOURS; n++)
       for (m = 0; m < NFRAMES; m++)
-	pixmaps[n][m] = NULL;
+        pixmaps[n][m] = NULL;
 
     first_run = FALSE;
   }
@@ -781,15 +805,6 @@ resize_graphics (void)
 gboolean
 configure_cb (GtkWidget * canvas, GdkEventConfigure * event)
 {
-  static gboolean first_run = TRUE;
-
-  if (first_run) {		/* Start the animation timer if necessary. */
-    /* 16 frames/second. */
-    g_timeout_add (62, (GSourceFunc) animation_timer, NULL);
-
-    first_run = FALSE;
-  }
-
   canvaswidget = canvas;
   drawing_area_width = event->width;
   resize_graphics ();
@@ -817,6 +832,8 @@ start_spinning (void)
     p->frame = 0;
     list++;
   }
+    
+  start_animation();
 }
 
 void
@@ -832,4 +849,6 @@ stop_spinning (void)
     p->style = ANI_SPINBACK;
     list++;
   }
+    
+  start_animation();
 }
